@@ -1,8 +1,23 @@
-# encoding: utf-8
+# Licensed to Elasticsearch B.V. under one or more contributor
+# license agreements. See the NOTICE file distributed with
+# this work for additional information regarding copyright
+# ownership. Elasticsearch B.V. licenses this file to you under
+# the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#  http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+
 # bootstrap.rb contains the minimal code to be able to launch Bundler to eventually be able
 # to retrieve the core code in the logstash-core gem which can live under different paths
 # depending on the launch context (local dev, packaged, etc)
-
 require_relative "bundler"
 require_relative "rubygems"
 require "pathname"
@@ -42,7 +57,7 @@ module LogStash
       !File.exists?(File.join(LogStash::Environment::LOGSTASH_HOME, "x-pack"))
     end
 
-    def windows?
+    def win_platform?
       ::Gem.win_platform?
     end
 
